﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Printing;
using System.Security.Cryptography;

/*
 * CodeInspector is basically a Ruby code verification tool which is to be used as the last step in
 * a Calc Process/Update or Dataflow operation, or it could be used at any time to inspect a document's
 * Ruby code file for the following:
 * 
 * A) Boxes referenced in Ruby code that are not on the document.
 * B) Unbalanced print statements - i.e. "print 'box1_1', box1_2" - where box references do not match.
 * C) Check Box Control code needs updating.
 * D) End of Data Flow comment is missing.
 * E) Usage of eval() - BNA Web support. We can use it in the new check box control method version.
 * F) name_block() usage - We are removing this from DataFlow to support BNA Web efforts.
 * G) Multiple methods with the same name in the same document - BNA Web support.
 * H) Method Name usage with signature code that differs from the backup master version - BNA web support.
 * 
 * This tool can be used as a stand-alone tool or it can be started from SciTE and used on a per document
 * basis.
*/
namespace CodeInspector
{
    public partial class Form1 : Form
    {
        public Form1(string[] args)
        {
            InitializeComponent();

            if (args.Length > 0)
            {
                fname = ProcessArgs(args);

                if (!String.IsNullOrEmpty(fname))
                {
                    doc_info = GetData(fname, no_comments);
                    ProcessData(doc_info);
                }
            }
        }

        // Event handler for menu item: File->Open:
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            fname = GetFileName();

            if (!String.IsNullOrEmpty(fname))
            {
                doc_info = GetData(fname, no_comments);
                ProcessData(doc_info);
            }
        }

        // Event handler for menu item: File->Exit:
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        // Event handler for menu item: Log->Print: We print the RichTextBox contents 'outText' as
        // plain text. User can select text or we default to all text.
        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (printDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    printDocument1.Print();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString(), "Error logging to printer.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void printDocument1_BeginPrint(object sender, System.Drawing.Printing.PrintEventArgs e)
        {
            char[] param = { '\n' };

            if (printDialog1.PrinterSettings.PrintRange == PrintRange.Selection)
            {
                lines = outText.SelectedText.Split(param);
            }
            else
            {
                lines = outText.Text.Split(param);
            }

            int i = 0;
            char[] trimParam = { '\r' };
            foreach (string s in lines)
            {
                lines[i++] = s.TrimEnd(trimParam);
            }
        }

        private int linesPrinted;
        private string[] lines;

        private void OnPrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            int x = e.MarginBounds.Left;
            int y = e.MarginBounds.Top;
            Brush brush = new SolidBrush(outText.ForeColor);

            while (linesPrinted < lines.Length)
            {
                e.Graphics.DrawString(lines[linesPrinted++],
                    outText.Font, brush, x, y);
                y += 15;
                if (y >= e.MarginBounds.Bottom)
                {
                    e.HasMorePages = true;
                    return;
                }
            }

            linesPrinted = 0;
            e.HasMorePages = false;
        }

        // Event handler for menu item: Log->Save: We save the RichTextBox contents 'outText' as
        // plain text.
        private void saveToFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (SaveFileDialog sfd = new SaveFileDialog())
            {

                // Initial directory is set to the Path [minus file name] of the file we are inspecting.
                if (!String.IsNullOrEmpty(fname))
                {
                    // We have already done exception handling for fname.
                    sfd.InitialDirectory = Path.GetDirectoryName(fname);
                }

                sfd.Title = "Save to Log File as Plain Text";

                if ((sfd.ShowDialog() == DialogResult.OK))
                {
                    try
                    {
                        outText.SaveFile(sfd.FileName, RichTextBoxStreamType.PlainText);
                    }
                    catch (IOException ex)
                    {
                        MessageBox.Show(ex.ToString(), "Error writing to log file.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private static ulong md5(byte[] Source)
        {
            if (Source.Length == 0)
                return 0;

            HashAlgorithm ha = GetHashAlgorithm("MD5");

            byte[] computed_hash = ha.ComputeHash(Source);

            ulong ret = BitConverter.ToUInt64(computed_hash, 0);

            return ret;
        }

        private static HashAlgorithm GetHashAlgorithm(string HashAlgorithmName)
        {
            switch (HashAlgorithmName.ToUpperInvariant())
            {
                case "MD5":
                    return MD5.Create();
                case "SHA1":
                    return SHA1.Create();
                case "SHA256":
                    return SHA256.Create();
                default:
                    throw new ArgumentOutOfRangeException("HashType", "Don't know how to create a '" + HashAlgorithmName + "' hash.");
            }
        }
    }
}
